searchData={"items":[{"type":"module","title":"ExUid2.Api.Sharing","doc":"This module provides the interface for the DSP to access the UID2 Operator Services's key sharing endpoint.","ref":"ExUid2.Api.Sharing.html"},{"type":"function","title":"ExUid2.Api.Sharing.fetch_keyring/0","doc":"Request the decryption keys necessary to decrypt UID2 tokens.","ref":"ExUid2.Api.Sharing.html#fetch_keyring/0"},{"type":"module","title":"ExUid2.Dsp","doc":"This is the client module for DSP-side decryption of UID2 tokens.\n\nBy default, when the `ExUid2` application starts, the `ExUid2.Dsp` server is started\nwith the `:active` mode, which refreshes the keyring every hour.\n\nTo manually control the keyring refresh calls, set the application\nwith the `runtime: false` option and start the `ExUid2.Dsp` server\nwith `ExUid2.Dsp.start_link(mode: :passive)`. Then, calling\n`ExUid2.Dsp.refresh/0` will make the query and refresh the keyring.","ref":"ExUid2.Dsp.html"},{"type":"function","title":"ExUid2.Dsp.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExUid2.Dsp.html#child_spec/1"},{"type":"function","title":"ExUid2.Dsp.decrypt_token/2","doc":"Attempts to decrypt a base64 encoded UID2 token.","ref":"ExUid2.Dsp.html#decrypt_token/2"},{"type":"function","title":"Example - ExUid2.Dsp.decrypt_token/2","doc":"iex> ExUid2.Dsp.decrypt(\"A3AAAAABI2RPtp1P7Fa66cRLOHzi2gkK2kxIEWpwX+cWgsLITmLS+/q7kHCuHMPhtweLapy0p8IXaR6T4eGlF3iloOSwzPaJ+PMUiRwdLVb8perCP4AmlnPeAlndOGAJNTlaSvqb1tUdUJwpOzkQv6yjE9LoLUT/82QhKt92WIehEdSjJm/YpSgLdMWazqXPzyJjTZ+GIgJn2k6qHb33AGoe5YSrrkp91xWL2H6Ziw==\")\n    {:ok,\n      %ExUid2.Uid2{\n        uid: \"ywsvDNINiZOVSsfkHpLpSJzXzhr6Jx9Z/4Q0+lsEUvM=\",\n        established_ms: 0,\n        site_id: 2,\n        site_key: %ExUid2.Keyring.Key{\n          activates_ms: 0,\n          created_ms: 0,\n          expires_ms: 1722448017227,\n          id: 2,\n          secret: <<32, 251, 7, 194, 132, 154, 250, 86, 202, 116, 104, 29, 131, 192,\n            139, 215, 48, 164, 11, 65, 226, 110, 167, 14, 108, 51, 254, 125, 65, 24,\n            23, 133>>,\n          keyset_id: nil\n        },\n        identity_scope: \"UID2\",\n        version: 3,\n        expires_ms: 1722448110794,\n        identity_type: :email\n      }}","ref":"ExUid2.Dsp.html#decrypt_token/2-example"},{"type":"function","title":"ExUid2.Dsp.refresh/0","doc":"Triggers a query to the Uid2 opperator server to get the latest keyring. Not required when the Dsp server is\nstarted in `:active` mode (which is the default) or when `ExUid2.Dsp.start_refresh_loop/0` has been called.","ref":"ExUid2.Dsp.html#refresh/0"},{"type":"function","title":"ExUid2.Dsp.start_link/1","doc":"","ref":"ExUid2.Dsp.html#start_link/1"},{"type":"function","title":"ExUid2.Dsp.start_refresh_loop/0","doc":"Starts hourly refreshing of the Keyring.\n\nWhen the `ExUid2.Dsp` server is started with `mode: :passive`, it won't periodically query the Uid2 operator server to\nrefresh the keyring. Calling `ExUid2.Dsp.start_refresh_loop/0` will refresh the keyring and make the `ExUid2.Dsp` server\nrefresh it again every hour.","ref":"ExUid2.Dsp.html#start_refresh_loop/0"},{"type":"module","title":"ExUid2.Encryption","doc":"Module responsible for performing encryption and decryption of Uid2 tokens.","ref":"ExUid2.Encryption.html"},{"type":"function","title":"ExUid2.Encryption.decrypt_token/3","doc":"","ref":"ExUid2.Encryption.html#decrypt_token/3"},{"type":"function","title":"ExUid2.Encryption.encrypt_token/4","doc":"","ref":"ExUid2.Encryption.html#encrypt_token/4"},{"type":"function","title":"ExUid2.Encryption.encrypt_v2_token/4","doc":"","ref":"ExUid2.Encryption.html#encrypt_v2_token/4"},{"type":"function","title":"ExUid2.Encryption.encrypt_v3_token/4","doc":"","ref":"ExUid2.Encryption.html#encrypt_v3_token/4"},{"type":"function","title":"ExUid2.Encryption.encrypt_v4_token/4","doc":"","ref":"ExUid2.Encryption.html#encrypt_v4_token/4"},{"type":"type","title":"ExUid2.Encryption.encryption_error/0","doc":"Encryption error atom.","ref":"ExUid2.Encryption.html#t:encryption_error/0"},{"type":"type","title":"ExUid2.Encryption.secret_key/0","doc":"","ref":"ExUid2.Encryption.html#t:secret_key/0"},{"type":"module","title":"ExUid2.Keyring","doc":"Struct holding the keys periodically fetched from the UID2 operator server.","ref":"ExUid2.Keyring.html"},{"type":"function","title":"ExUid2.Keyring.get_key/2","doc":"","ref":"ExUid2.Keyring.html#get_key/2"},{"type":"function","title":"ExUid2.Keyring.new/1","doc":"","ref":"ExUid2.Keyring.html#new/1"},{"type":"type","title":"ExUid2.Keyring.t/0","doc":"Struct holding the keys periodically fetched from the UID2 operator server.\n\n* `:keys` - The list of available keys\n\n* `:info` - Other information provided via the UID2 operator server's `/v2/keys/sharing` endpoint","ref":"ExUid2.Keyring.html#t:t/0"},{"type":"module","title":"ExUid2.Keyring.Info","doc":"Struct holding information about the keyring.","ref":"ExUid2.Keyring.Info.html"},{"type":"function","title":"ExUid2.Keyring.Info.new/1","doc":"","ref":"ExUid2.Keyring.Info.html#new/1"},{"type":"type","title":"ExUid2.Keyring.Info.t/0","doc":"","ref":"ExUid2.Keyring.Info.html#t:t/0"},{"type":"module","title":"ExUid2.Keyring.Key","doc":"Struct holding the information for a given key.","ref":"ExUid2.Keyring.Key.html"},{"type":"function","title":"ExUid2.Keyring.Key.new/1","doc":"","ref":"ExUid2.Keyring.Key.html#new/1"},{"type":"type","title":"ExUid2.Keyring.Key.t/0","doc":"Struct holding the information for a given key.\n\n* `:activates_ms` - Time when the key becomes active (Unix timestamp in milliseconds)\n\n* `:created_ms` - Time when the key was created (Unix timestamp in milliseconds)\n\n* `:id` - The key's ID for lookups in the keyring\n\n* `:secret` - That key's secret that must be used to decrypt the UID2 tokens.\n\n* `keyset_id` - Unknown","ref":"ExUid2.Keyring.Key.html#t:t/0"},{"type":"module","title":"ExUid2.Uid2","doc":"Struct holding the decrypted and parsed UID2 token.","ref":"ExUid2.Uid2.html"},{"type":"function","title":"ExUid2.Uid2.new/1","doc":"","ref":"ExUid2.Uid2.html#new/1"},{"type":"type","title":"ExUid2.Uid2.t/0","doc":"Struct holding the decrypted and parsed UID2 token.\n\n* `:uid` - User's unique ID. This should always be a 44 bytes base64-encoded hashed ID.\n\n* `:established_ms` - The time when the token was first created (Unix timestamp in milliseconds)\n\n* `:site_id` - The site's ID used for finding the right decryption key.\n\n* `site_key` - The key found in the Keyring for the given `site_id`\n\n* `identity_scope` - The identity scope found in the keyring (should probably always be \"UID2\").\n\n* `identity_type` - Whether this is an hashed email or phone number. Will be `:unknown` for V2 tokens.\n\n* `version` - The advertising token version.\n\n* `expires_ms` - The time after which the token will be expired (Unix timestamp in milliseconds).","ref":"ExUid2.Uid2.html#t:t/0"},{"type":"extras","title":"ExUid2","doc":"# ExUid2\n\nA library to interact with [Unified ID2](https://unifiedid.com/docs/intro)\n\nIt currently only handles the DSP part of UID2 (decrypting UID2 tokens in bid requests).","ref":"readme.html"},{"type":"extras","title":"Installation - ExUid2","doc":"Add `:ex_uid2` to the list of dependencies in mix.exs\n\n```elixir\ndef deps do\n  [\n      {:ex_uid2, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExUid2","doc":"Add the ExUid2 configuration to your application's config file:\n\n```elixir\nimport Config\n\nconfig :ex_uid2,\n  base_url: \"https://uid2_operator_server.com\",\n  api_key: \"your_api_key\",\n  secret_key: \"your_secret_key\"\n\n```\n\nOnce started, the application will periodically request a fresh keyring from the configured UID2 operator server. Encrypted tokens\ncan then be decrypted.\n\n```elixir\nExUid2.Dsp.decrypt_token( )\n{:ok,\n %ExUid2.Uid2{\n   uid:  ,\n   established: ~U[2024-07-02 01:17:56.501Z],\n   site_id: 11,\n   site_key: %ExUid2.Keyring.Key{\n     activates: 1717269873,\n     created: 1717183473,\n     expires: 1725909873,\n     id: 2383,\n     secret:  ,\n     keyset_id: nil\n   },\n   identity_scope: \"UID2\",\n   identity_type: :email,\n   version: 3,\n   expires: ~U[2024-07-06 19:12:10.313Z]\n }}\n```\n\nIf the keyring hasn't been properly fetched by the time a token decryption is attempted, an error tuple will be returned instead:\n\n```elixir\nExUid2.Dsp.decrypt_token( )\n{:error, :no_keyring_stored}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Token decryption specification - ExUid2","doc":"Since no specification for the token decryption protocol could be found, the decryption\nprotocol has been guessed by reverse engineering the existing SDKs. It may include errors\nand misconceptions, so this specification is likely to be erroneous or incomplete. However, it has been\nshown to work for decoding encrypted UID2 tokens in a production environment.","ref":"readme.html#token-decryption-specification"},{"type":"extras","title":"General flow - ExUid2","doc":"1. A keyring is periocically updated by querying the `/v2/key/sharing` endpoint.\n2. The encrypted token envelope is fetched from the bid request.\n3. The token is base64 decoded.\n4. The envelope is parsed to extract the token version and other info necessary for decryption.\n5. The master key ID parsed from the envelope is used to get the master key from the keyring.\n6. The master payload is decrypted.\n7. The decrypted master payload is parsed to extract the expiration timestamp and other info necessary to decrypt the identity\n8. The expiration timestamp is checked to ensure the token is still valid.\n9. The site key ID parsed from the master payload is used to get the site key from the keyring.\n10. The indentity payload is decrypted.\n\n### `/v2/key/sharing` request\nWhile the `/v2/key/sharing` endpoint is undocumented, The workflow to query UID2's endpoints is documented, the \n[Encrypting Requests and Decrypting Responses](https://unifiedid.com/docs/getting-started/gs-encryption-decryption#encryption-and-decryption-code-examples) documentation seem to apply there. It is what \nthis library is doing.","ref":"readme.html#general-flow"},{"type":"extras","title":"V2 tokens - ExUid2","doc":"1. The token is base64 decoded using the standard mode to get the envelope.\n2. The envelope is parsed to extract the token version, the master key ID, the master IV and the master payload\n3. The master key ID parsed from the envelope is used to get the master key from the keyring.\n4. The master payload is decrypted using AES-256-CBC, the master key and the master IV.\n5. The decrypted master payload is parsed to extract the expiration timestamp, the site key ID, the site IV and the identity payload.\n6. The expiration timestamp is checked to ensure the token is still valid.\n7. The site key ID parsed from the master payload is used to get the site key from the keyring.\n8. The indentity payload is decrypted using AES-256-CBC, the site key and the site IV.\n9. The decrypted identity payload is parsed to extract the site ID, the user ID length, the user ID binary and the established timestamp.\n\n#### V2 Encrypted token envelope\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 1            | UID2 token version |\n| 1              | 4            | Master key ID. Must be uint32 big endian. |\n| 5              | 16           | Master 128-bit initialization vector (IV), which is used to randomize data encryption. |\n| 21             | N            | Master Payload (encrypted) |\n\n#### V2 Decrypted Master Payload\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 8            | Expiration time in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 8              | 4            | Site key ID. Must be uint32 big endian. |\n| 12             | 16           | Identity 128-bit initialization vector (IV), which is used to randomize data encryption. |\n| 28             | N            | Identity Payload (encrypted) |\n\n#### V2 Decrypted Identity Payload\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 4            | Site ID. Must be uint32 big endian. |\n| 4              | 4            | Length in bytes of ID binary. Must be uint32 big endian. |\n| 8              | N            | Base64-encoded ID binary (the actual user ID). |\n| 8 + N          | 4            | Unknown. |\n| 12 + N         | 8            | Time when the identity was established in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 20 + N         | M            | Unknown. |","ref":"readme.html#v2-tokens"},{"type":"extras","title":"V3 Tokens - ExUid2","doc":"1. The token is base64 decoded using the standard mode to get the envelope.\n2. The envelope is parsed to extract the token version, the identity type, master key ID and the master payload.\n3. The master key ID parsed from the envelope is used to get the master key from the keyring.\n4. The master payload is parsed and its data decrypted using AES-256-GCM and the master key.\n5. The decrypted master payload is parsed to extract the expiration timestamp, the site key ID, and the identity payload.\n6. The expiration timestamp is checked to ensure the token is still valid.\n7. The site key ID parsed from the master payload is used to get the site key from the keyring.\n8. The indentity payload is parsed and its data decrypted using AES-256-GCM and the site key.\n9. The decrypted identity payload is parsed to extract the site ID, the user ID binary and the established timestamp.\n\n#### V3 Encrypted token envelope\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 1            | Identity type byte. |\n| 1              | 1            | Version byte.  |\n| 2              | 4            | Master Key ID. Must be uint32 big endian. |\n| 6              | N            | Master Payload (encrypted) |\n\n#### V3 Decrypted Master Payload\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 8            | Expiration time in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 8              | 8            | Timestamp when the token was generated in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 16             | 4            | Operator Site ID (unused) |\n| 20             | 1            | Operator Type (unused) |\n| 21             | 4            | Operator Version (unused) |\n| 25             | 4            | Operator Key ID (unused) |\n| 29             | 4            | Site key ID. Must be uint32 big endian. |\n| 33             | N            | Identity Payload (encrypted) |\n\n#### V3 Decrypted Identity Payload\n| Offset (bytes) | Size (bytes) | Description |\n| -------------- | ------------ | ----------- |\n| 0              | 4            | Site ID. Must be uint32 big endian. |\n| 4              | 8            | Publisher ID. Must be uint64 big endian. |\n| 12             | 4            | Client Key ID (unused) |\n| 16             | 4            | Privacy Bits (unused) |\n| 20             | 8            | Time when the identity was established in milliseconds (Unix timestamp). Must be uint64 big endian. |\n| 28             | 8            | Unknown. |\n| 36             | N            | Base64-encoded ID binary (the actual user ID). |","ref":"readme.html#v3-tokens"},{"type":"extras","title":"V4 tokens - ExUid2","doc":"V4 tokens are exactly like V3 tokens, but are base64 encoded using the `urlsafe` mode.","ref":"readme.html#v4-tokens"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}